/*
 * jSite2 - UserInterface.java -
 * Copyright © 2008 David Roden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package net.pterodactylus.jsite.core;

/**
 * Interface definition for user interfaces.
 *
 * @author David ‘Bombe’ Roden &lt;bombe@freenetproject.org&gt;
 */
public interface CoreListener {

	//
	// project configuration
	//

	/**
	 * Notifies a listener that loading the projects finished successfully.
	 *
	 * @param directory
	 *            The directory the nodes were loaded from
	 */
	public void loadingProjectsDone(String directory);

	/**
	 * Notifies all listeners that loading the projects has failed.
	 *
	 * @param directory
	 *            The directory the projects were tried to load from
	 * @param throwable
	 *            The exception that occured while saving, if any
	 */
	public void loadingProjectsFailed(String directory, Throwable throwable);

	/**
	 * Notifies a listener that the projects were successfully saved to the
	 * given directory.
	 *
	 * @param directory
	 *            The directory the projects were saved to
	 */
	public void savingProjectsDone(String directory);

	/**
	 * Notifies a listener that saving the projects has failed.
	 *
	 * @param directory
	 *            The directory the projects were to be saved to
	 * @param throwable
	 *            The exception that occured when saving the projects, if any
	 */
	public void savingProjectsFailed(String directory, Throwable throwable);

	/**
	 * Notifies a listener that the given project was added.
	 *
	 * @param project
	 *            The project that was added.
	 */
	public void projectAdded(Project project);

	/**
	 * Notifies a listener that a project was cloned.
	 *
	 * @param clonedProject
	 *            The clones project
	 * @param projectClone
	 *            The clone of the project
	 */
	public void projectCloned(Project clonedProject, Project projectClone);

	/**
	 * Notifies a listener that the given project was removed.
	 *
	 * @param project
	 *            The project that was removed.
	 */
	public void projectRemoved(Project project);

	//
	// node configuration
	//

	/**
	 * Notifies a listener that the nodes were successfully loaded.
	 *
	 * @param directory
	 *            The directory the nodes were loaded from
	 */
	public void loadingNodesDone(String directory);

	/**
	 * Notifies a listener that loading the nodes has failed.
	 *
	 * @param directory
	 *            The directory the nodes were loaded from
	 * @param throwable
	 *            The exception that occured while loading the nodes
	 */
	public void loadingNodesFailed(String directory, Throwable throwable);

	/**
	 * Notifies a listener that the nodes were successfully saved.
	 *
	 * @param directory
	 *            The directory the nodes were saved to
	 */
	public void savingNodesDone(String directory);

	/**
	 * Notifies a listener that saving the nodes has failed.
	 *
	 * @param directory
	 *            The directory the nodes were saved to
	 * @param throwable
	 *            The exception that occured while saving the nodes
	 */
	public void savingNodesFailed(String directory, Throwable throwable);

	//
	// basic core functionality
	//

	/**
	 * Notifies all listeners that the core has loaded.
	 */
	public void coreLoaded();

	/**
	 * Notifies a listener that the core was stopped.
	 */
	public void coreStopped();

	//
	// node stuff
	//

	/**
	 * Notifies a listener that a node was added to the core.
	 *
	 * @param node
	 *            The node that was added.
	 */
	public void nodeAdded(Node node);

	/**
	 * Notifies a listener that a node was removed from the core. Before a node
	 * is removed, it will be disconnected (and
	 * {@link #nodeDisconnected(Node, Throwable)} will be called).
	 *
	 * @param node
	 *            The node that was removed
	 */
	public void nodeRemoved(Node node);

	/**
	 * Notifies all listeners that the core started connecting to the given
	 * node. Before a node is connected, it will be added (and
	 * {@link #nodeAdded(Node)} will be called).
	 *
	 * @param node
	 *            The node that is being connected
	 */
	public void nodeConnecting(Node node);

	/**
	 * Notifies all listeners that the core connected to the given node.
	 *
	 * @param node
	 *            The node that is connected
	 */
	public void nodeConnected(Node node);

	/**
	 * Notifies a listener that a connection to a node has failed.
	 *
	 * @param node
	 *            The node that could not be connected
	 * @param cause
	 *            The cause of the failure
	 */
	public void nodeConnectionFailed(Node node, Throwable cause);

	/**
	 * Notifies all listeners that the core disconnected from the given node.
	 *
	 * @param node
	 *            The node that was diconnected
	 * @param throwable
	 *            The exception that caused the disconnect, or <code>null</code>
	 *            if there was no exception
	 */
	public void nodeDisconnected(Node node, Throwable throwable);

	//
	// project insertion stuff
	//

	/**
	 * Notifies a listener that a project insert started.
	 *
	 * @param project
	 *            The project that is now inserting
	 */
	public void projectInsertStarted(Project project);

	/**
	 * Notifies a listener that a project insert made progress.
	 *
	 * @param project
	 *            The project that made progress
	 * @param totalBlocks
	 *            The number of total blocks
	 * @param requiredBlocks
	 *            The number of required blocks
	 * @param successfulBlocks
	 *            The number of successfully inserted blocks
	 * @param failedBlocks
	 *            The number of blocks that could not be inserted
	 * @param fatallyFailedBlocks
	 *            The number of blocks that can not be inserted even with
	 *            retries
	 * @param finalizedTotal
	 *            <code>true</code> if the number of total blocks is already
	 *            finalized, <code>false</code> otherwise
	 */
	public void projectInsertProgressed(Project project, int totalBlocks, int requiredBlocks, int successfulBlocks, int failedBlocks, int fatallyFailedBlocks, boolean finalizedTotal);

	/**
	 * Notifies a listener that a project insert has generated a URI.
	 *
	 * @param project
	 *            The project that generated a URI
	 * @param uri
	 *            The URI that was generated
	 */
	public void projectInsertGeneratedURI(Project project, String uri);

	/**
	 * Notifies a listener that a project insert has finished.
	 *
	 * @param project
	 *            The project that has finished
	 * @param success
	 *            <code>true</code> if the project was inserted successfully,
	 *            <code>false</code> otherwise
	 */
	public void projectInsertFinished(Project project, boolean success);

}
