/*
 * jSite2 - Insert.java - Copyright © 2008 David Roden
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package net.pterodactylus.jsite.core;

/**
 * Represents a currently running or past insert.
 *
 * @author David ‘Bombe’ Roden &lt;bombe@freenetproject.org&gt;
 */
public class Insert extends Request {

	/** The project that was inserted. */
	private final Project project;

	/** The start time of the insert. */
	private final long startTime;

	/** The end time of the insert. */
	private long endTime;

	/**
	 * Creates a new insert that starts now.
	 *
	 * @param project
	 *            The project that is inserted
	 * @param node
	 *            The node the project is inserted to
	 * @param identifier
	 *            The identifier of the insert request
	 */
	public Insert(Project project, Node node, String identifier) {
		this(project, node, identifier, System.currentTimeMillis());
	}

	/**
	 * Creates a new insert.
	 *
	 * @param project
	 *            The project that is inserted
	 * @param node
	 *            The node the project is inserted to
	 * @param identifier
	 *            The identifier of the insert request
	 * @param startTime
	 *            The time the insert was started
	 */
	public Insert(Project project, Node node, String identifier, long startTime) {
		super(node, identifier);
		this.project = project;
		this.startTime = startTime;
	}

	/**
	 * Returns the project that is inserted.
	 *
	 * @return The inserted project
	 */
	public Project getProject() {
		return project;
	}

	/**
	 * Returns the start time of the insert.
	 *
	 * @return The start time of the insert
	 */
	public long getStartTime() {
		return startTime;
	}

	/**
	 * Returns the end time of the insert. If the insert has not yet finished,
	 * <code>-1</code> is returned.
	 *
	 * @return The end time of the insert, or <code>-1</code> if the insert is
	 *         still running
	 */
	public long getEndTime() {
		return endTime;
	}

	/**
	 * Sets the end time of the insert.
	 *
	 * @param endTime
	 *            The end time of the insert
	 */
	public void setEndTime(long endTime) {
		this.endTime = endTime;
	}

}
